
#version 120
//this is needed For bitwise operations
#extension GL_EXT_gpu_shader4 : enable


//internal constants
	const float shadowBias = 0.000045;
	const float pi = 3.142;


//inputs from code
	uniform float ambientLight;
	uniform float useLight;
	
	uniform float Scale;
	
	uniform vec4 lightPosRot;
	uniform vec4 lightPos;
	uniform vec4 lightLookat;
	uniform vec4 lightColor;
	
	uniform float shadowRange;
	uniform float shadowFalloff;

	uniform float time;

	// this is the light shadow map
		uniform sampler2D ShadowMap;

	// this is the input texturemap
		uniform sampler2D TextureMap;



//inputs from vertex shader
	varying vec4 vertexDepth;
//	varying vec4 vertexSmooth;

	varying vec3 rotate;

	varying vec4 vertexPos;
	varying vec4 vertexRaw;
	varying vec4 vertexNormal;
	
//	varying vec4 objectCenter;

	varying vec4 cameraPosRot;

//	varying vec4 lightPosRot;
		
	varying vec4 vertex2light;
	varying vec4 vertex2camera;

	varying vec4 lightNormal;
	varying vec4 eyeNormal;

	varying vec4 ShadowCoord;

	varying vec3 shineColor;

	varying float isLight;
	varying float isGlass;
	varying float isFauxLight;

	varying float materialShine;
	varying float materialReflect;

	varying float materialType;
	varying float materialData1;
	varying float materialData2;
	varying float materialData3;
	varying float materialData4;
	varying float materialSmooth;
	varying float materialInvert;
	varying float materialRaise;
	varying float materialFade;
	varying float materialBump;
	varying float materialAlpha;

	varying float materialSAlpha;
	varying float materialSSS;
	varying float materialSSScatter;
	varying float materialSSBounce;

	//texture mapping
		varying float width;
		varying float height;
	
		varying float overlay;
		varying float xpos;
		varying float ypos;
		
		varying float overlay1;
		varying float xpos1;
		varying float ypos1;
	
		varying float cellWidth;
		varying float cellHeight;

		varying float cellWidth2;
		varying float cellHeight2;

	//normal calculation
		varying vec3 v_V;
		varying vec3 v_P;
		
	//debugging
		varying vec4 debugCol;
		
//global variables
	vec4 color;
	vec4 color2;
	vec4 color3;

	vec3 diffuse;
	vec3 diffuse2;
	float alpha;
	float shadow;
	float surface;
	float glow;
	float specReflect;





float length2(vec2 p)
{
	return dot(p, p);
}

float noise(vec2 p){
	return fract(sin(fract(sin(p.x) * (4231.13311)) + p.y) * 3131.0011);
}

float worley(vec2 p) {
	float d = 1e30;
	for (int xo = -1; xo <= 1; ++xo) {
		for (int yo = -1; yo <= 1; ++yo) {
			vec2 tp = floor(p) + vec2(xo, yo);
			d = min(d, length2(p - tp - vec2(noise(tp))));
		}
	}
	return 3.0*exp(-4.0*abs(2.0*d - 1.0));
}

float fworley(vec2 p, float tm) {
	return sqrt(sqrt(sqrt(
		1.6 * // light
		worley(p*32. + 4.3 + tm *.25) *
		sqrt(worley(p * 64. + 5.3 + tm * -0.625)) *
		sqrt(sqrt(worley(p * -100. + 9.3))))));
}





float semirand(vec2 co, float xx, float yy)
{
	float x = fract( sin( dot(co.xy ,vec2(12.9898, 78.233)) ) * xx);
	float y = fract( cos( dot(co.yx ,vec2(58.9898, 18.233)) ) * yy);
	return (x + y) * 0.1;
}




float randz(vec2 v){
	return fract(sin(dot(v.xy,vec2(33.9898,78.233))) * 43758.5453);
}





vec2 hash2(vec2 uv) {
	return fract(sin(mat2(15.23, 35.48, 74.26, 159.37) * uv) * 47658.23);
}





float rand(vec2 co){
  return fract( sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float noiser(vec2 co)
{
	vec2 x,y;
	x.x = floor(co.x);
	x.y = ceil(co.x);
	y.x = floor(co.y);
	y.y = ceil(co.y);
	
	return mix(
		mix(rand(vec2(x.x, y.x)), rand(vec2(x.y, y.x)), 1.0 - (x.y - co.x)),
		mix(rand(vec2(x.x, y.y)), rand(vec2(x.y, y.y)),  1.0 - (x.y - co.x)),
		1.0 - (y.y - co.y)
	);
}

float fbm(vec2 co)
{
	float x;
	x += noiser(co);
	x += noiser(  2.0 * co)  /  2.0;
	x += noiser(  4.0 * co)  /  4.0;
	x += noiser(  8.0 * co)  /  8.0;
	x += noiser( 16.0 * co)  /  5.0;
	x += noiser( 32.0 * co)  /  6.0;
	return x / 1.96875;
}




float Arand(vec2 n) { 
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float Anoise(vec2 n) {
	const vec2 d = vec2(0.0, 1.0);
	vec2 b = floor(n), f = smoothstep(vec2(0.0), vec2(1.0), fract(n));
	return mix(mix(Arand(b), Arand(b + d.yx), f.x), mix(Arand(b + d.xy), Arand(b + d.yy), f.x), f.y);
}

float Afbm(vec2 n){//, float total, float amplitude) {
	float total = 0.0;
	float amplitude = 1.0;
	for (int i = 0; i < 7; i++) {
		total += Anoise(n) * amplitude;
		n += n;
		amplitude *= 0.5;
	}
	return total;
}





float undulate(float x)
{
	if(x < -0.4){
		return 0.15 + 2.857 * (x + 0.75)*(x + 0.75);
	}else if(x < 0.4){
		return 0.95 - 2.8125 * (x*x);
	}
	
	return 0.26 + 2.666 * (x - 0.7)*(x - 0.7);
}

float marble(float p, float PI)
{
	float a = 2.0;

	// this computes 3 octaves of turbulence
	float t1 = 0.25 * p;

	float m = undulate(sin(p * 4.0 * PI * t1));
	return m;
}






vec2 ripple(vec2 o, vec2 p, float f, float tm){
	float d = distance(p, o);
	vec2 offset = vec2(0);
	offset.x = asin((p.x-o.x)/d)/3.1415 * sin(d*f-tm);
	offset.y = asin((p.y-o.y)/d)/3.1415 * sin(d*f-tm);
	return offset;
}




vec4 textureRND2D(vec2 uv){
	uv = floor(fract(uv)*1e3);
	float v = uv.x+uv.y*1e3;
	return fract(1e5*sin(vec4(v*1e-2, (v+1.)*1e-2, (v+1e3)*1e-2, (v+1e3+1.)*1e-2)));
}

float noisex(vec2 p) {
	vec2 f = fract(p*1e3);
	vec4 r = textureRND2D(p);
	f = f*f*(3.0-2.0*f);
	return (mix(mix(r.x, r.y, f.x), mix(r.z, r.w, f.x), f.y));	
}

float cloud(vec2 p) {
	float v = 0.0;
	v += noisex(p*1.)*.80000;
	v += noisex(p*2.)*.25000;
	v += noisex(p*4.)*.12500;
	v += noisex(p*8.)*.06250;
	v += noisex(p*16.)*.03125;
	return v*v*v;
}





float hashb2(vec2 uv) {
	return fract(sin(uv.x * 15.78 + uv.y * 35.14) * 43758.23);
}



float field(vec3 p, float tm) {
	float strength = 4. + 0.01 * log(1.e-6 + fract(sin(tm) * 373.11));
	float accum = 0.;
	float prev = 0.;
	float tw = 0.;
	for (int i = 0; i < 16; ++i) {
		float mag = dot(p, p);
		p = abs(p) / mag + vec3(-0.2 + (sin(tm * 0.01) * 0.01), -.4, -1.445-sin(p)+sin(p/0.01));
		float w = exp(-float(i) / 8.);
		accum += w * exp(-strength * pow(abs(-mag - prev/50.), 1.45));
		tw += w;
		prev = mag;
	}
	return max(0., 5. * accum / tw - 0.7);
}




float halfLambert(in vec3 vect1, in vec3 vect2)
{
    float product = dot(vect1,vect2);
    return product * 0.5 + 0.5;
}
 
float blinnPhongSpecular(in vec3 normalVec, in vec3 lightVec, in float specPower)
{
    vec3 halfAngle = normalize(normalVec + lightVec);
    return pow(clamp(0.0,1.0,dot(normalVec,halfAngle)),specPower);
}





float randsn( float x, float y ){return fract( sin( x + y*0.1234 )*1234.0 );}

float interpolar(vec2 cord, float L){
	float XcordEntreL= cord.x/L;
	float YcordEntreL= cord.y/L;
	float XcordEnt=floor(XcordEntreL);
	float YcordEnt=floor(YcordEntreL);
	float XcordFra=fract(XcordEntreL);
	float YcordFra=fract(YcordEntreL);
	float l1 = randsn(XcordEnt, YcordEnt);
	float l2 = randsn(XcordEnt+1.0, YcordEnt);
	float l3 = randsn(XcordEnt, YcordEnt+1.0);
	float l4 = randsn(XcordEnt+1.0, YcordEnt+1.0);
	float inter1 = (XcordFra*(l2-l1))+l1;
	float inter2 = (XcordFra*(l4-l3))+l3;
	float interT = (YcordFra*(inter2 -inter1))+inter1;
	return interT;
}




//use= bump_normal(X,Y, N, 0..1)
vec3 bump_normal(vec3 dPdx, vec3 dPdy, vec3 fragNormal, float value)
{
	vec2 dV = vec2(dFdx(value), dFdy(value));
	
//	vec3 dPdx = dFdx(fragVertex);
//	vec3 dPdy = dFdy(fragVertex);  
	
	vec3 dPdz = normalize(fragNormal);
	dPdy = normalize(cross(dPdz, dPdx));
	dPdx = normalize(cross(dPdy, dPdz));
	
	vec3 N = normalize(-dV.x * dPdx - dV.y * dPdy + dPdz);
	return N;
}

/*
vec3 GetNormal()
{
	vec3 X = dFdy(fragVertexEc);
	vec3 Y = dFdx(fragVertexEc);
	vec3 flatNormalEc = normalize( cross(X, Y) );
	//
	// Measure the rate of change of the value as we move in x and y.
	// Then measure the rate of change of the position in eye coordinates
	// as we move in x and y, so that we can re-express the value change (dV) in
	// eye coordinates - and thus create a new eye coordinate normal.
	//
	vec2 dV = vec2(dFdx(value), dFdy(value));
	vec3 dPdx = dFdx(fragVertexEc);
	vec3 dPdy = dFdy(fragVertexEc);  
	 
	//
	// dPdx is effectively the vector one unit in X as measured in eye space;
	// thus to do height mapping, we bump up the "height" (i.e. direction of
	// the fragment normal) by the relative change in value of the height as
	// we move one unit in X (i.e. dV.x).  Repeat the same for Y.  Now we have
	// a "tilted" surface plane to compute a new normal from.
	//
	vec3 T = dPdx + dV.x * fragNormalEc;
	vec3 B = dPdy + dV.y * fragNormalEc;
	vec3 N = normalize( cross(T, B) );
	 
	//
	// But...
	//
	// The new bump-based normal was computed relative to the plane of the
	// triangle: i.e. a flat shaded normal.  We'll assume flat and smooth
	// normal are 'relatively similar' and use a simple linear delta between the
	// flat and smooth normal to adjust to the bump normal onto the "smooth"
	// surface of the object.
	//
	vec3 delta = fragNormalEc - normalize(cross(dPdx, dPdy));
	 
	return normalize(N + delta);
}
*/


void main()
{
	int matType = int(materialType);
	surface = 1.0;
	glow = 0.0;

	//we use depth as it is correctly reotated with the normals - pos is not correctly rotated
//	vec3 X = dFdx(vec3(vertexDepth.x, vertexDepth.y, -vertexDepth.z));
//	vec3 Y = dFdy(vec3(vertexDepth.x, vertexDepth.y, -vertexDepth.z));
	vec3 X = dFdx(vertexDepth.xyz);
	vec3 Y = dFdy(vertexDepth.xyz);

	vec3 N = normalize(cross(X, Y));

//		vec3 VN = N;//vec4(-vertexRaw.x, 0.5-vertexRaw.y, -vertexRaw.z, 1);

//		float cry = cos(rotate.y);
//		float sry = sin(rotate.y);
//		N.x = VN.x * cry - VN.z * sry;
//		N.z = VN.x * sry + VN.z * cry;


	N = mix(N, vertexNormal.xyz, materialSmooth);

//	diffuse = N;
//	diffuse = vertexNormal.xyz;
//	diffuse = vec3(1,1,1) * vertexRaw.x;
//	diffuse = vec3(1,1,1) * vertexNormal.x;
//	diffuse = vec3(vertexRaw.x, vertexNormal.x, 0);
//	diffuse = vec3(objectCenter.x,objectCenter.y,1) ;
	
//	gl_FragColor = vec4(diffuse, 1);
//	return;



	//this edges things - v nice
//	float width = length(vec2(length(X),length(Y)));
//	diffuse =  vec3(width*materialData1, length(X)*materialData1, length(Y)*materialData1);

//	float b = (length(X) + length(Y)) * materialSmooth;
//	vec3 N = normalize(cross(X + b, Y + b));

//	N = mix(N, vertexSmooth.xyz , materialSmooth);
//	N = mix(N, sN , materialSmooth);

//	vec3 lightDir = normalize( vec3(lightPos));
	vec3 lightDir = -vec3(lightPos-vertexDepth);
//	float lambert = max( 0.0, dot( N, lightDir ) );
//	diffuse = vec3(1,1,1) * lambert;
	//diffuseIntensity is lambert - declared lower down
//	diffuseIntensity = max( 0.0, dot( N, lightDir));
	
//	diffuse = lightDir;
	
//	shineIntensity *= diffuseIntensity;
//	diffuse = vec3(1,1,1) * diffuseIntensity;	
//	diffuse = vec3(1,1,1) * shineIntensity;
//	diffuse = shineColor;
//	diffuse = N;
//	diffuse = fN;
//	diffuse = delta;
//	diffuse = vertexPos.xyz;
//	diffuse = objectCenter.xyz;
//	diffuse = vertexSmooth.xyz;
//	diffuse = vertexPos.xyz;

//	vec3 w = (abs(X) + abs(Y)) * 100;
//	diffuse = w;
//	diffuse = vec3(1,1,1) * length(X)+length(Y);
	


	vec3 V = normalize(v_V);
	vec3 R = reflect(V, N);
	vec3 L = normalize(lightPosRot.xyz);
	vec3 H = normalize(L+V);
	vec3 E = normalize(-vertexPos.xyz);
	
	vec2 UV = gl_TexCoord[0].xy;

	vec3 nvertex2light = normalize(vertex2light.xyz);

	vec4 finalCol = vec4(1,1,1,1);

//			diffuse = finalCol.rgb;
     

	//show the top facing	
//	diffuse = vec3(1,1,1) * N.g;

	//basic diffuse:
//	diffuse = vec3(1,1,1) * max(dot(L, N), 0.0);

//	float smooth = (sin(UV.x*pi) * sin(UV.y * pi)) * materialSmooth;
//	float smooth = 1.0-((sin(UV.x*pi)) * materialSmooth);
	float smooth = 1.0-( cos(UV.y*pi) * materialSmooth);
	smooth *= 1.0-((sin(UV.x*pi)) * materialSmooth);
	
//	diffuse = vec3(1,1,1) * smooth;
	
	//basic specular
//	diffuse = vec3(1,1,1) * pow(max(dot(R, L), 0.0), materialShine);
	
//	diffuse = texture2D(TextureMap, gl_TexCoord[0].xy).rgb;

//		diffuse = N;
//	gl_FragColor = vec4(diffuse, 1);
//	return;

	
	//bypass is texture 0 has not been set
	diffuse = gl_Color.rgb;
	float finalShine = materialShine;
	if (overlay != 1){
		alpha = gl_Color.a;
	}else{
		
		vec3 colormono;

	//get the texture mapped color
		color = gl_Color;
		alpha = color.a;
	
		//base image
			vec2 uv = vec2(gl_TexCoord[0].x * width, gl_TexCoord[0].y * height);
			vec2 uv1;
			uv1.x = xpos + (uv.x * cellWidth);
			uv1.y = ypos + (uv.y * cellHeight);

			color = texture2D(TextureMap, uv1);
			float alpha2 = color.a;

			if (materialData4 > 0){			
				float sh = ((color.r+color.g+color.b)/3.0);
				finalShine = sh * materialShine * (2.0-(materialData4 * 0.01));
			}

		//mono baseimage
			if (materialType == 0){
				//mono
				float m = (color.r + color.g + color.b) / 3.0;
				colormono = vec3(m, m, m);
				colormono = mix(color.rgb, colormono, materialData1*0.01);
				color = vec4( colormono, alpha2);
			}
			
		//secondary image
			if (overlay1 == 1){
				uv1.x = xpos1 + (uv.x * cellWidth2);
				uv1.y = ypos1 + (uv.y * cellHeight2);
	
				color2 = texture2D(TextureMap, uv1);
				float al = color.a;
				color2.a = mix(color2.a, 0, materialData2*0.01);
				if (color2.a > color.a) al = color2.a;
				
				color = vec4(mix(color.rgb, color2.rgb, color2.a).rgb, al);
			}
	
//		diffuse = color.rgb;
		
		color2 = vec4(color.r * diffuse.r, color.g * diffuse.g, color.b * diffuse.b, color.a * alpha);
	
		diffuse = color2.rgb;
		alpha = color2.a;
	}
	
	float reflect = materialReflect;

	if(materialType < 16){//0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
		if(materialType < 8){//0,1,2,3,4,5,6,7
			
			if(materialType < 4){ //0,1,2,3
				if (materialType < 2){ //0,1
					if (materialType < 1){ // 0 none

//						float m = (diffuse.r + diffuse.g + diffuse.b) / 3.0;
//						diffuse = mix(diffuse, gl_Color.rgb * m, materialData1 * 0.01);
//						diffuse = mix(diffuse, gl_Color.rgb, materialFade);
//						surface = m * materialData3 * 0.01;

						float m = (diffuse.r + diffuse.g + diffuse.b) / 3.0;
						diffuse = mix(diffuse, gl_Color.rgb, materialFade);
						surface = m * materialData3 * 0.01;

						diffuse2 = diffuse;
						
					}else{ //1 water

						UV = mix(vertexRaw.zx, gl_TexCoord[0].xy+vertexRaw.zx, materialData4*0.01);
						vec2 p = UV * 8.0 - vec2(30.0);
						vec2 i = p;
						float c = 1.0;
						float inten = (materialData3 * 0.0002); // toned down intensity
						
						float MAX_ITER = materialData1;
						
						for (int n = 0; n < MAX_ITER; n++){
							// forms a fractal of sort. t is slightly higher for every iteration,
							// in range: -2.0, -0.5, 0.0, +0.25, +0.4, +0.5, slowly approaches 1.0
							float t = time * (1.0 - 3.0 / float(n+1)) * (materialData2 * 0.04);
						
							// try just plain i=p; to see what the effect is. It's recursively distorting the grid coordinates.
							i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
						
							// rewrote and removed the +t from i.x, i.y.
							c += 1.0/length(p.xy / (vec2(sin(i.x), cos(i.y)) / inten));
						}

						c /= float(MAX_ITER); // the thing accumulates and then averages.
						// removed few of the effects achieved by pow(1.5 - sqrt(c), 7.0) and scaled it down a bit more.
						c = 1.0-sqrt(c);
						c = pow(c, 0.75);
						c += c*c;
						surface = c;
						diffuse2 = diffuse * c;// + vec3(0.1, 0.15, 0.25);

					}
				}else{ //2,3
					if (materialType < 3){ //2 checker
						float BOX_SIZE = materialData1 / 100.0;
						int evenRow = int(mod(UV.y / BOX_SIZE, 2.0));
						if( int(mod(UV.x / BOX_SIZE, 2.0)) == evenRow ){
							diffuse2 = diffuse;//vec3(1.0, 0.0, 0.0);
						}else{
							diffuse2 += mix(vec3(1,1,1), vec3(-1,-1,-1), materialData3 * 0.01);
							diffuse2 = mix(diffuse2, diffuse, materialData2 * 0.01);
							surface = 0;
						}
						
					}else{ //3 Velvet

						float width = length(vec2(length(X),length(Y)));
						diffuse =  diffuse * vec3(width*materialData1, length(X)*materialData1, length(Y)*materialData1);

						float base = materialData1 * 0.01 * dot(L, V);
						
						float rim = (materialData2 * 0.01) * 1.0 - dot(R, -L);
						float undu = materialData3 * 0.01 * undulate(X.x);
						vec3 vBase = vec3(1,1,1) * clamp(base, 0,1);
						vec3 vRim = vec3(1,1,1) * clamp(rim, 0,1);
						vec3 vUndu = vec3(1,1,1) * clamp(undu, 0,1);
						
						diffuse2 = diffuse * (vBase + vRim + vUndu);
					}
				}
				
			}else{ //4,5,6,7
			
				if (materialType < 6){ //4,5
					if (materialType < 5){ //4 oil
						
						UV = mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01);
						vec2 p = UV * 8.0 - vec2(30.0);
						vec2 i = p;
						float c = 1.0;
						float inten = (materialData3 * 0.0002); // toned down intensity
						
						float MAX_ITER = materialData1;
						
						for (int n = 0; n < MAX_ITER; n++){
							// forms a fractal of sort. t is slightly higher for every iteration,
							// in range: -2.0, -0.5, 0.0, +0.25, +0.4, +0.5, slowly approaches 1.0
							float t = time * (1.0 - 3.0 / float(n+1)) * (materialData2 * 0.04);
						
							// try just plain i=p; to see what the effect is. It's recursively distorting the grid coordinates.
							i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
						
							// rewrote and removed the +t from i.x, i.y.
							c += 1.0/length(p.xy / (vec2(sin(i.x), cos(i.y)) / inten));
						}

						c /= float(MAX_ITER); // the thing accumulates and then averages.
						// removed few of the effects achieved by pow(1.5 - sqrt(c), 7.0) and scaled it down a bit more.
						c = 1.0-sqrt(c);
						c = pow(c, 0.75);
						c += c*c;
						
						diffuse2 = vec3(cos(p.x+p.y+c), cos(p.y)+sin(c+p.y), (sin(p.x+p.y+c)+cos(p.x+p.y))*c);

						surface = (diffuse2.r + diffuse2.g);
						if (surface < 0.5){
							surface = 0;
						}
						
					}else{ //5 caustic
/*
						UV = vec2(gl_TexCoord[0].x * materialData2 * 0.1, gl_TexCoord[0].y * materialData3 * 0.1);
						UV = UV + (UV * 0.5);
						
						float t = cos(UV.x) * sin(UV.y);
						t = fworley(UV * materialData1 * 0.01, t);
						t *= exp(-length2(abs(2.0 * UV - 1.0)));
						surface = t;
						diffuse2 = diffuse * (t+0.2);
*/
						float tm = time * materialData2 * 0.1;
						UV = mix(gl_TexCoord[0].xy, mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01), materialData3 * 0.01);

						vec2 p = UV * .05 + .5;
						vec3 c = vec3(.1, .1, .1);
						float c1 = cloud(p*.3 + tm * .0008)*.5;
						c1 = pow(c1,1.5);
						float c2 = cloud(p*.2 + tm * .00002)*.75;
						c2 = pow(c2, 1.5);
						c.rgb += vec3(.0, .0, .0) * c1*.5;
						p += c1*0.006125;
						p -= c2*0.006125;
						c.gbr += vec3(0., 0., 0.) * c2*.5;
						//p.y *= c.g;
						c.grb += vec3(1., 1., 1.) * cloud(p*.1+time*-.00005)*1.;
						c *= c * 2;//.725;
						
						c += (materialData1 * 0.01) - 0.5;

						surface = (c.r + c.g + c.b) / 3;

						diffuse2 = diffuse * c * c;
						surface = 2.0-clamp(((c.r + c.g + c.b) / 3),0,4);
//						glow = surface;

					}
				}else{ //6,7
					if (materialType < 7){ //6 lava
						
						float tm = time * materialData2 * 0.1;
						UV = mix(gl_TexCoord[0].xy, mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01), materialData3 * 0.01);

						vec2 p = UV * .05 + .5;
						vec3 c = vec3(.1, .1, .1);
						float c1 = cloud(p*.3 + tm * .0008)*.5;
						c1 = pow(c1,1.5);
						float c2 = cloud(p*.2 + tm * .00002)*.75;
						c2 = pow(c2, 1.5);
						c.rgb += vec3(.0, .0, .0) * c1*.5;
						p += c1*0.006125;
						p -= c2*0.006125;
						c.gbr += vec3(0., 0., 0.) * c2*.5;
						//p.y *= c.g;
						c.grb += vec3(1., 1., 1.) * cloud(p*.1+time*-.00005)*1.;
						c *= c * 2;//.725;
						
						c += (materialData1 * 0.01) - 0.5;

						surface = (c.r + c.g + c.b) / 3;

						diffuse2 = diffuse * c * c;
						surface = -((c.r + c.g + c.b) / 3);
						glow = -surface;

					}else{ //7 distress
					
						float tm = materialData1 * 8;
						UV = vec2(gl_TexCoord[0].x * materialData2 *4, gl_TexCoord[0].y * materialData3 * 1);
						
//						vec3 p = vec3(UV.xy, UV.y) * vec3(sin(tm / 200.0), cos(tm / 120.0),  sin(tm / 18) + cos(tm / 18));
						vec3 p = vec3(UV.xy, UV.y) * vec3(sin(tm*0.001), cos(tm / 120.0),  sin(tm) * cos(tm / 18));
//						p = 0.3 * vec3(sin(tm / 200.0), cos(tm / 120.0),  sin(tm / 18) + cos(tm / 18));
//						float t = field(p, tm);
//						float t = p.x;
						
						diffuse2 = vec3(1,1,1) * p;
//						diffuse2 = diffuse * (t+t);
						surface = (p.x+p.y+p.z)*0.3332;
					}
				}			
			}
			
		}else{ //8,9,10,11,12,13,14,15
			if(materialType < 12){ //8,9,10,11
				if (materialType < 10){ //8,9
					if (materialType < 9){ // 8 metal
						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);

						float d = length(UV.xy - (materialData1 * 0.1));
//						float g = clamp(sin(d * materialData2 * 0.01), 0.0, 1.0);
						
						float diff = d * dot(L,N);
						float spec = diff + pow(dot(N,H), (materialData2 * 0.01));
	
						diffuse2 = diffuse * vec3(spec, spec, spec);
	
						float sur = (materialData3 * 0.2);
						surface = sin(UV.x * sur) * cos(UV.y * sur);
						
					}else{ //9 fabric
						float spec = cos(UV.x * (materialData1+1) * 10);
						float spec2 = cos(UV.y * (materialData2+1) * 10);
						spec = ((spec + spec2) * 0.5) * ((100-materialData3) * 0.02);
						surface = spec - ((100-materialData3) * 0.02) + 1;
						diffuse2 = diffuse * (1-(vec3(spec, spec, spec)/4));
		
						float diff = materialReflect * dot(L,N);
	//					float specu = clamp(diff + pow(dot(N,H), (materialShine / materialReflect)), 0, 1);
						float specu = diff + pow(dot(N,H), ((finalShine+1) / materialReflect));
						diffuse2 += vec3(specu, specu, specu);
//						doSurface = 0;
						surface *= 3;
					}
				}else{ //10,11
					if (materialType < 11){ //10 marble
						float PI = 3.1415 * ((materialData3+1) * 0.44);
						float val = marble((gl_TexCoord[0].x * ((materialData1+1) * 0.01) ) + (gl_TexCoord[0].y * ((materialData2+1)* 0.01) ), PI);
						diffuse2 = diffuse * (vec3(val, val, val) + 0.4);
						surface = val * 3;
						
					}else{ //11 diagonals
						float r = semirand(gl_TexCoord[0].xy, materialData1 * 0.1, materialData2 * 0.1);
						float oag = dot(vec3(0.33333), H);
						r *= (oag + 2.0) * ((materialData3+1) * 0.1);
						surface = r;
						diffuse2 = vec3(r, r, r);
						surface *= 3;
					}
				}
			}else{ //12,13,14,15
				if (materialType < 14){ //12,13
					if (materialType < 13){ //12 damage
						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);

						float d = length(UV.xy - (materialData1 * 0.01));
						float g = clamp(sin(d * materialData2 * 0.01), 0.0, 1.0);
						
						float diff = d * dot(L,N);
						float spec = diff + g * (materialData2 * 0.1) * d;

						float oag = 1.0-clamp(dot(vec3(materialData1*0.01), H) * materialData3 * 0.01, 0, 1);
						surface = spec * diff * oag;
						diffuse2 = diffuse * surface;
						
					}else{ //13 glow
						float intensity = (materialData1 / 25);
						float cutoff = 1.0 - (materialData2 * 0.01);
						float falloff = materialData3 * 0.01;
						
						diffuse2 = diffuse;
						
						if (diffuse2.x > cutoff){
							float red = ((diffuse2.x + 0.02) - cutoff) * intensity;
							if (red < 0) red = 0;
							diffuse2.x += red;
						}
						if (diffuse2.y > cutoff){
							float red = ((diffuse2.y + 0.02) - cutoff) * intensity;
							if (red < 0) red = 0;
							diffuse2.y += red;
						}
						if (diffuse2.z > cutoff){
							float red = ((diffuse2.z + 0.02) - cutoff) * intensity;
							if (red < 0) red = 0;
							diffuse2.z += red;
						}
	
						if (diffuse2.x < falloff){
							diffuse2.x = diffuse.x - (falloff-diffuse2.x) * intensity;
						}
						if (diffuse2.y < falloff){
							diffuse2.y = diffuse.y - (falloff-diffuse2.y) * intensity;
						}
						if (diffuse2.z < falloff){
							diffuse2.z = diffuse.z - (falloff-diffuse2.z) * intensity;
						}
						
					}
				}else{ //14,15
					if (materialType < 15){ //14 cloth
						float NdotL = 0.5 + 0.5 * dot( N, L );
						float VdotN = 0.5 + 0.5 * dot( V, N );
						float scale = 1.0-(materialData1 * 0.01);

						float rim = scale * pow( VdotN, (materialData2 * 0.1) );

						float inner = scale * pow( 1-NdotL, (materialData3 * 0.01) );
						
						diffuse2 = diffuse * rim + inner;
						diffuse2 *= diffuse2;
						
						surface = NdotL;
						
					}else{ //15 ripple
//						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						UV = mix(gl_TexCoord[0].xy, mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01), materialData3 * 0.01);
						
						float tm = time * materialData1;
					
						vec2 dir = UV - vec2(.5);
						float dist = distance(UV, vec2(0.5));
						vec2 offset = ripple(vec2(-0.2, 0.4), UV, 50.0, tm);
						offset += ripple(vec2(0.4, -0.6), UV, 50.0, tm);
						offset += ripple(vec2(0.1, 0.2), UV, 40.0, tm);
						offset += ripple(vec2(-0.2, -0.6), UV, 40.0, tm);
						offset += ripple(vec2(0.3, 0.3), UV, 40.0, tm);
						offset += ripple(vec2(-0.4, -0.2), UV, 40.0, tm);
						UV += offset/40.0;
							
						float col = 1.0-(offset.y * (materialData2 * 0.01));
						surface = col;
						
						diffuse2 = diffuse * col;
					}
				}			
			}
		}
		
	}else{ //16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
	
		if(materialType < 24){//16,17,18,19,20,21,22,23
			if(materialType < 20){ //16,17,18,19
				if (materialType < 18){ //16,17
					if (materialType < 17){ // 16 cells
//						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						UV = mix(gl_TexCoord[0].xy, mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01), materialData3 * 0.01);
						UV *= (materialData1);//4.0;

						vec2 g = floor(UV);
						vec2 f = fract(UV);
						vec2 v = hash2(g) - f;
						float d = length(v);
						d = (1.0 - d) * (materialData2 * 0.01);

						surface = (d * 2) - 1;

						diffuse2 = diffuse * d;
			
					}else{ //17 clear water
						float count = materialData1 * 0.04;
						UV = mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01);
						vec2 p = UV * 8.0- vec2(30.0);
						vec2 i = p;
						float c = 1.0;
						float inten = .05;
					
						float tm = time * materialData2 * 0.1;
						for (int n = 0; n < count; n++){
							float t = tm * (1.0 - (3.0 / float(n+1)));
							i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
							c += 1.0/length(vec2(p.x / (sin(i.x+t)/inten),p.y / (cos(i.y+t)/inten)));
						}
						c /= count;
						c = 1.5-sqrt(c);
						surface = c*c;
						diffuse2 = vec3(pow(c, 7.0)) + vec3(0.0, 0.15, 0.25) * 3;
						diffuse2 = mix(diffuse2, diffuse * diffuse2, materialData3 * 0.01);
					}
				}else{ //18,19
					if (materialType < 19){ //18 cloud
						float tm = time * materialData2 * 0.1;
//						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						UV = mix(gl_TexCoord[0].xy, mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01), materialData3 * 0.01);

						vec2 p = UV * .05 + .5;
						vec3 c = vec3(.1, .1, .1);
						float c1 = cloud(p*.3 + tm * .0008)*.5;
						c1 = pow(c1,1.5);
						float c2 = cloud(p*.2 + tm * .00002)*.75;
						c2 = pow(c2, 1.5);
						c.rgb += vec3(.0, .0, .0) * c1*.5;
						p += c1*0.006125;
						p -= c2*0.006125;
						c.gbr += vec3(0., 0., 0.) * c2*.5;
						//p.y *= c.g;
						c.grb += vec3(1., 1., 1.) * cloud(p*.1+time*-.00005)*1.;
						c *= 2;//.725;

						diffuse2 = c;
						surface = c.r;

					}else{ //19 lightning
						float tm = time * materialData2 * 4;
						float count = (materialData1 + 10) * 0.1;

						vec2 uPos = UV;
						uPos.x -= 0.0;
						uPos.y -= 0.5;
						
						vec3 color = vec3(0.0);
						float vertColor = 0.0;
						const float k = 3.;
						for( float i = 1.0; i < count; ++i ){
							float t = tm * (2.0);
						
							uPos.y += sin( uPos.x*i - t) * 0.055;
							float frequency = abs(1.0/(120.0*k) / uPos.y);
							vertColor += frequency;
							color += vec3(frequency*(i*0.9), frequency*i/k, pow(frequency,0.6)*.2 );
						}
						
						vec4 color_final = vec4(color, 1.0);
						gl_FragColor = color_final;
						float ft = fract(tm * 3);
						vec3 finalColor = gl_FragColor.rgb;
						finalColor += vec3( rand( UV +  112.+ ft ), rand( UV +  159.+ ft ), rand( UV + 111.+ ft ) ) / 32.0;	
						
						float d = distance((gl_TexCoord[0].yy), vec2(0.5,0.5) * (materialData3 * 0.01));
						finalColor *= smoothstep(0.75, 0.0, d) * 5;
//						gl_FragColor.rgb = finalColor;

						diffuse2 = diffuse + finalColor;
						alpha = finalColor.r;

					}
				}
			}else{ //20,21,22,23
				if (materialType < 22){ //20,21
					if (materialType < 21){ //20 sunburst

						UV = gl_TexCoord[0].xy;
						UV -= 0.5;
						float c = cos(UV.x * (materialData1 * 0.1)) * cos(UV.y * (materialData1 * 0.1)) ;
						
						float d = 1.0-(materialData3*0.02);
						
						c = c*c * materialData2 * 0.2;
						
						c = c+d;
						
						diffuse2 = diffuse * (c*c);//vec3(c,c,c);//diffuse + vec3(c, c, c) - vec3(d,d,d);
						surface = c;

					}else{ //21 irridescent
						float tm = time * materialData2;
						float scl = materialData1 * 0.01;
						UV *= 1.0-(materialData3 * 0.01);
					
						float color = UV.x*16.0+sin(UV.y*8.0+tm* scl);
						diffuse2 = vec3( 0.5+0.5*sin(color+tm),0.5+scl*cos(color+tm),0.5+scl*sin(color+tm+3.1415));
						surface = color;

					}
				}else{ //22,23
					if (materialType < 23){ //22 soft edge

//						vec3 v = normalize(V);
//						vec3 l = normalize(L);
						float vdotn = dot(V,N);
						float ldotn = dot(L,N);
						float cos_theta_i = ldotn;
						float theta_r = acos(vdotn);
						float theta_i = acos(cos_theta_i);
						float cos_phi_diff = dot(normalize(V-N*vdotn), normalize(L-N*ldotn));
						float nalpha = max(theta_i,theta_r);
						float nbeta = min(theta_i,theta_r);
						
						float r = materialData1 * 0.03;
						float a = 1.0-0.5 * r / (r+0.33);
						float b = 0.45 * r / (r+0.09);
						
						if (cos_phi_diff >= 0){    
							b *= sin(nalpha) * tan(nbeta);
						}else{
							b=0.0;
						}
						
						float o = 1.0-(cos_theta_i * (a+b));
						float non = o * materialData2 * 0.01;
						o += non + non;
						
						float bl = materialData3 * 0.01;
						float oh = (o * bl) - (bl*0.5);
						
						vec3 diffuse3 = diffuse * 4;

//						diffuse2 = (diffuse * o * o) + oh;
						diffuse2 = ((diffuse3 * o * o) + oh);
//						diffuse2 = vec3(1,1,1) * o;

						
					}else{ //23 blocks

						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						UV.x *= materialData1;
						UV.y *= materialData2;
						
						float c = hashb2(floor(UV));
						
						diffuse2 = vec3(c,c,c);
						surface = c;

					}
				}			
			}
		}else{ //24,25,26,27,28,29,30,31
			if(materialType < 28){ //24,25,26,27
				if (materialType < 26){ //24,25
					if (materialType < 25){ //24 blobs
						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						UV *= (materialData2 * 0.1);
						
						float c = 0.0;
						for(int i = 0; i < materialData1/4; i++){
							float f = float(i);
							UV += 
							c += max(0.5 - length(vec2(rand(vec2(f, 1.0)), rand(vec2(-1.0, f))) * 2.0 - 1.0 - UV), 0.0);
						}
						surface = c * 2;
						diffuse2 = diffuse * c;
						
					}else{ //25 fire
						const vec3 c1 = vec3(0.1, 0.0, 0.0);
						const vec3 c2 = vec3(0.7, 0.0, 0.0);
						const vec3 c3 = vec3(0.2, 0.0, 0.0);
						const vec3 c4 = vec3(1.0, 0.9, 0.0);
						const vec3 c5 = vec3(0.1);
						const vec3 c6 = vec3(0.9);
						
						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						float tm = time * materialData1 * 4;
						UV *= (materialData2 * 0.1);
						
						float q = Afbm(UV - tm * 0.1);
						vec2 r = vec2(Afbm(UV + q + tm * 0.7 - UV.x - UV.y), Afbm(UV + q - tm * 0.4));
						vec3 c = mix(c1, c2, Afbm(UV + r)) + mix(c3, c4, r.x) - mix(c5, c6, r.y);
						
						surface = cos(1.57 * UV.y);
						surface *= surface;
						diffuse2 = c * surface;
						surface = ((diffuse2.r+diffuse2.g+diffuse2.b)/3) * materialBump;

						glow = clamp(q, 0, 1);
					}
				}else{ //26,27
					if (materialType < 27){ //26 wave
						vec2 center = vec2(0.5,0.5);
						
						UV = mix(gl_TexCoord[0].xy, vec2((vertexRaw.z), (vertexRaw.x)), materialData3 * 0.01);
						float tm = time * materialData1;
						UV *= (materialData2 * 0.1);

						vec3 col = vec3(UV, 0.5 + 0.5 * sin(tm));
								
						float x = (center.x - UV.x);
						float y = (center.y - UV.y);
							
						//float r = -sqrt(x*x + y*y); //uncoment this line to symmetric ripples
						float r = -(x * x + y * y);
						float z = 1.0 + 0.5 * sin((r +  tm)/0.013);
						
						surface = z;
						diffuse2 = col*z*diffuse;
						
					}else{ //27 movement
						UV = gl_TexCoord[0].yx;//vec2((vertexRaw.z), (vertexRaw.x));
						UV *= (materialData2);
						
						float tm = time * 3 * materialData1;
						tm += materialData4 * 0.1;

						float c = cos(UV.y + tm);
						surface = c  * ((1.0-gl_TexCoord[0].y) * materialData3 * 0.2);
						diffuse2 = diffuse * c  * ((1.0-gl_TexCoord[0].y) * materialData3 * 0.2);
					}
				}
			}else{ //28,29,30,31
				if (materialType < 30){ //28,29
					if (materialType < 29){ //28 warp

//						float Pi = 4.14159 * materialData1 * 0.01;
					float color = 0.0;
					float color2 = 0.0;

					float tm = materialData2 * 0.01;
					UV = vec2((vertexRaw.z), (vertexRaw.x)+tm);
					UV *= (materialData3) ;
					
					for ( int i = 0; i < materialData1; i++ ){
						float p = fract(float(i) / float(N) - .05 );
						float q = fract( float(i) / float(N) - .04999 );
						float a = p * (1.0-p);
						color += a * (interpolar(gl_FragCoord.xy-UV/2., UV.x/pow(2.0, p*p*float(N)))-.5);
						color2 += a * (interpolar(gl_FragCoord.xy-UV/2., UV.y/pow(2.0, q*q*float(N)))-.5);
						surface = a  * ((1.0-gl_TexCoord[0].y) * materialData3 * 0.2);
					}
					color += .5;
					vec3 out1 = color * vec3(.5,.0,1.0);
					vec3 out2 = color2 * vec3(0.4,1.0,0.0);
					
					diffuse2 = out1-out2;

					}else{ //29 bands

						float Pi = 4.14159 * materialData1 * 0.01;
						UV = mix(gl_TexCoord[0].xy*Pi, vec2(vertexRaw.z+vertexRaw.y*Pi, vertexRaw.x-Pi), materialData3 * 0.01);
						UV *= (materialData2 * 0.1);

						surface = noise(UV / 100000);
						diffuse2 = diffuse * surface;
						surface *= 4;
					}
				}else{ //30,31
					if (materialType < 31){ //30 height

						float Pi = (materialData1+1) * 0.0001;
						UV = mix(gl_TexCoord[0].xy*Pi, vec2(vertexRaw.z, vertexRaw.x), Pi) * Pi;

						float dark = clamp(vertexRaw.y*vertexRaw.y, 0, 10) / ((materialData2 *0.01)+ 1);
						
//						diffuse2 = diffuse * dark;
//						diffuse2 = vec3(1,1,1) * noisex(vec2(UV.x *0.001, UV.y * 0.01)) * 
						surface = noise(UV) * noise(vec2(UV.y+Pi, UV.x-Pi))-Pi;
						diffuse2 = mix(diffuse*dark, diffuse * noise(UV) * noise(vec2(UV.y+Pi, UV.x-Pi))-Pi, materialData3 * 0.01);

					}else{ //31 ice

						diffuse = mix(vec3(0.01,0.01,0.5), vec3(0.4,0.7,0.9), materialData1 * 0.01);

//						UV = vertexRaw.zx;
						UV = mix(vertexRaw.zx, vertexPos.zx, materialData4*0.01);
						vec2 p = UV * 8.0 - vec2(30.0);
						vec2 i = p;
						float c = 1.0;
						float inten = (materialData3 * 0.0002); // toned down intensity
						
						float MAX_ITER = 12 + (materialData1 * 0.5);
						float dark = clamp(vertexRaw.y*vertexRaw.y, 0, 10) / ((materialData2 *0.01)+ 1);
						
						for (int n = 0; n < MAX_ITER; n++){
							float t = (1.0 - 3.0 / float(n+1)) * (materialData2 * 0.04);
						
							i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
						
							c += 1.0/length(p.xy / (vec2(sin(i.x), cos(i.y)) / inten));
						}

						c /= float(MAX_ITER); // the thing accumulates and then averages.
						// removed few of the effects achieved by pow(1.5 - sqrt(c), 7.0) and scaled it down a bit more.
						c = 1.0-sqrt(c);
						glow = c * dark;
						c = pow(c, 0.75);
						c += c*c;
						surface = c;
						diffuse2 = mix(diffuse, diffuse * c, dark);// + vec3(0.1, 0.15, 0.25);
						
//						diffuse2 = N;//vec3(1,1,1) * N.g;

					}
				}			
			}
		}
	}

	diffuse = clamp(mix(diffuse2, diffuse, materialFade), 0, 1);
//	diffuse = vec3(1,1,1);

	float baseSurface = mix(surface, 1.0-surface, materialInvert) + materialRaise;
	surface = baseSurface + (baseSurface * (1-(vertexDepth.z*0.3)) * 2);

	vec3 diffuseglow = clamp(diffuse * glow,0,2)*2;

	shadow = 1.0;

//	gl_FragColor = vec4(shineColor, alpha);
//	return;
	
	if (materialAlpha > 0){
		if (materialSAlpha > 0){
			 if(materialSAlpha < baseSurface){
//				diffuse = vec3(1,1,0);// *materialAlpha;
//				gl_FragColor = vec4(diffuse,1);
//				return;
				discard;
			}
		}else if (materialSAlpha > -1){
			 if(materialSAlpha+1 > baseSurface){
//				diffuse = vec3(1,1,0);// *materialAlpha;
//				gl_FragColor = vec4(diffuse,1);
//				return;
				discard;
			}
		}
			float m = (diffuse.r+diffuse.g+diffuse.b)/3;
			if(m < materialAlpha){
				discard;
			}
	}

	float bumpSurface = baseSurface * materialBump;
	N = normalize(bump_normal(X,Y, N, bumpSurface*3));
//	N = bump_normal(X,Y, N, smooth + (surface * materialBump));

//	float lambert = max( 0.0, dot( N, lightDir ) );
//	diffuse = vec3(1,1,1) * lambert;
//	diffuse = vec3(1,1,1) * baseSurface;
//	gl_FragColor = vec4(diffuse,1);
//	return;
//	gl_FragColor = vec4(-N.x, -N.y, N.z,1);
//	return;
//	gl_FragColor = vec4(vertexDepth.xy,0,1);
//	return;
//	gl_FragColor = vec4(vertexDepth.zzz*0.075,1);
//	return;

	//this is the diffuse output
	//diffuse
	//this is the normal output
	//vec3(-N.x, -N.y, N.z)
	//this is the x,y position output
	//vertexDepth.xy
	//this is the depth output
	//vertexDepth.z*0.075

	float lightFacing = dot(N, lightDir);
	float diffuseIntensity = max( 0.0, lightFacing);

//	if (useLight > 0.5 && isLight < 0.5) {
		//this is the base color
//		vec3 ambient = diffuse * ambientLight;
	float finalReflect = materialReflect;
	if (useLight > 0.5){
		//this is the base color
		vec3 ambient = diffuse;
		if (isLight < 0.5){
			ambient *= ambientLight;
		}else{
//			diffuse += ambient;
			finalReflect += materialReflect;
			diffuseglow += (ambient*finalReflect);
			glow += finalReflect;
		}

		//now the diffuse light
		if (diffuseIntensity > 0){
			diffuse = mix(ambient, diffuse, diffuseIntensity * lightColor.rgb * materialReflect * 0.5 * lightColor.a);
		}else{
			diffuse = ambient;
		}


		if (finalShine > 0 && lightFacing > 0.5){
//			diffuse *= dot(lightPosRot.xyz, N);
//			gl_FragColor = vec4(lightColor.rgb, 1);
//			return;	

			//basic grey finish for testing
//			diffuse = vec3(1,1,1) * diffuseIntensity;//ambientLight;

//			blinnPhongSpecular(in vec3 normalVec, in vec3 lightVec, in float specPower)
//			vec3 realLightDir = normalize(  lightPosRot.xyz );
//			float shineCutoff = clamp(blinnPhongSpecular(N, realLightDir, materialRaise*1000), 0,1);
			float shineCutoff = clamp(blinnPhongSpecular(N, lightDir, materialRaise*1000), 0,2);
			diffuse += clamp(shineColor * shineCutoff * finalShine * surface * 25, 0,1);
		}

			if (materialSSS > 0){
//				float materialSSBounce = materialData1 * 0.03;
//				float materialSSScattering = materialData3 * 0.1;

				float attenuation = 10.0 * (1.0 / distance(lightPosRot,vertexPos));
				vec3 eVec = normalize(-vertexDepth.xyz);
				vec3 lVec = normalize(lightPosRot.xyz-vertexDepth.xyz);
			 
				vec4 dotLN = vec4(halfLambert(lVec, N) * attenuation);
				dotLN *= vec4(diffuse.rgb,1);
			
				vec3 indirectLightComponent = vec3(materialSSBounce * max(0.0,dot(-N,lVec)));
				indirectLightComponent += materialSSBounce * halfLambert(-eVec,lVec);
				indirectLightComponent *= attenuation * materialReflect;
			 
				vec3 rim = vec3(1.0 - max(0.0,dot(N,eVec)));
				rim *= rim;
				rim *= max(0.0,dot(N,lVec)) * shineColor.rgb;
			 
				finalCol = dotLN + vec4(indirectLightComponent,1.0);
				finalCol.rgb += (rim * materialSSScatter * attenuation * finalCol.a);
				finalCol.rgb += vec3(blinnPhongSpecular(N,lVec, finalShine) * attenuation * shineColor * finalCol.a * 0.05);
				finalCol.rgb *= lightColor.rgb * lightColor.a;
			}
//		diffuse *= finalCol.rgb;
		diffuse = mix(diffuse, (ambient+diffuse)*finalCol.r, materialSSS);

//		diffuse = N;
//		diffuse = vec3(1,1,1) * surface;
//		diffuse = vec3(1,1,1) * 1-(vertexDepth.z*0.3);
	}	

	//this is the amount of light being emitted
//	gl_FragColor = vec4((vec3(1,1,1) * glow), 1);
	//this is the amount of light being reflected
//	gl_FragColor = vec4((vec3(1,1,1) * finalReflect), 1);
//	return;

	gl_FragColor = vec4(diffuse + diffuseglow, alpha);
}